"""compare two or more vcf files
================================

At the moment a naive implementation of a multi-way comparison.
Only compare chromosome and position and remove all variants that
are PASS or ".".

"""

import os
import sys
import re
import pysam
import pandas
import cgatcore.experiment as E
import cgatcore.iotools as iotools


def read_vcf_positions_into_dataframe(filename, filters=None):

    vcf_in = pysam.VariantFile(filename)

    if filters is None:
        filters = []

    pass_filter = False
    snp_filter = False

    for f in filters:
        if f == "PASS":
            pass_filter = True
        elif f == "SNP":
            snp_filter = True

    records = []
    c = E.Counter()
    for record in vcf_in:
        c.input += 1
        f = record.filter.keys()
        if pass_filter and "PASS" not in f and "." not in f:
            c.removed_pass_filter += 1
            continue
        if snp_filter:
            is_snp = (len(record.ref) == 1 and
                      len(record.alts) == 1 and
                      len(record.alts[0]) == 1)
            if not is_snp:
                c.removed_snp_filter += 1
                continue

        c.output += 1
        records.append((record.chrom, record.pos))

    df = pandas.DataFrame.from_records(
        records,
        columns=["chrom", "pos"])

    E.info("{}: {}".format(filename, c))

    return df


def main(argv=None):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "--regex-filename", dest="regex_filename", type="string",
        help="extract column name from filename via regular expression "
        "[%default]")

    parser.add_option(
        "--filter", dest="filters", type="choice", action="append",
        choices=("PASS", "SNP"),
        help="apply filters to VCFs when reading "
        "[%default]")

    parser.set_defaults(
        regex_filename=None,
        filters=[],
    )

    (options, args) = E.start(parser,
                              argv=argv,
                              add_output_options=True)

    if len(args) < 2:
        raise ValueError("requiring at least 2 input filenames")

    dfs = []
    for filename in args:
        if options.regex_filename:
            try:
                name = re.search(options.regex_filename, filename).groups()[0]
            except AttributeError:
                raise ValueError("regular expression '{}' does not match {}".format(
                    options.regex_filename, filename))
        else:
            name = iotools.snip(os.path.basename(filename), ".vcf.gz")

        E.debug("reading data from {}".format(filename))
        df = read_vcf_positions_into_dataframe(filename,
                                               filters=options.filters)
        df[name] = 1
        dfs.append(df)

    ndata = len(dfs)
    merged_df = dfs[0]
    for df in dfs[1:]:
        merged_df = merged_df.merge(df, how="outer")
    merged_df = merged_df.fillna(0)
    ddf = merged_df.drop(["chrom", "pos"], axis=1)
    set_counts = ddf.groupby(by=list(ddf.columns)).size()
    set_counts = set_counts.reset_index()
    set_counts.columns = list(set_counts.columns[:-1]) + ["counts"]

    set_counts.to_csv(options.stdout,
                      sep="\t",
                      index=False)
    E.stop()


if __name__ == "__main__":
    sys.exit(main())



'''
script_template.py
=============================================

:Tags: Python

Purpose
-------

Convert the output of a metaphlan analysis to a preferred table format


Usage
-----

Example::

   python metaphlan2table.py --help

Type::

   python metaphlan2table.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import optparse
import cgat.Metaphlan as Metaphlan
import cgatcore.experiment as E


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser(
        version="%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $",
        usage=globals()["__doc__"])

    parser.add_option("-t", "--sequence-type", dest="type", type="choice",
                      choices=("read_map", "rel_ab"), help="type of file to be parsed to a table")

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv)

    assert options.type, "must specify infile type"
    if options.type == "read_map":
        options.stdout.write(
            "seq_id\tkingdom\tphylum\tclass\torder\tfamily\tgenus\tspecies\n")
        for entry in Metaphlan.read_map_iterator(sys.stdin):
            options.stdout.write("\t".join(
                [entry.seq_id, entry.kingdom, entry.phylum, entry.c_lass, entry.order, entry.family, entry.genus, entry.species]) + "\n")

    elif options.type == "rel_ab":
        options.stdout.write("taxon_level\ttaxon\trel_abundance\n")
        for entry in Metaphlan.relative_abundance_iterator(sys.stdin):
            options.stdout.write(
                "\t".join([entry.taxon_level, entry.taxon, entry.abundance]) + "\n")

    # write footer and output benchmark information.
    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''
fastq2tpm.py - use rapid/lightweight alignment RNA seq quantification methods
=============================================================================

Purpose
-------

.. Wrapper for kallisto & sailfish, need to add in Salmon
NB: I wrote this before Tom implemented his mapper-class based
approach, so that should supersede this.  I just need to get round to
actually doing it.

Usage
-----

.. This might change, for now usage is limited to creating an index with either
Kallisto or Sailfish and quantifying from fastq files.

Example::

   python fastq2tpm.py

Type::

   python fastq2tpm.py --help

for command line help.

Command line options
--------------------

'''

import sys
import cgatcore.experiment as E
import os
import subprocess
import re


# ------------------------------------------------------- #
# Functions for expression quantification
# ------------------------------------------------------- #

def runSailfishIndex(fasta_file, outdir, threads,
                     kmer):
    '''
    Wrapper for sailfish index
    '''

    if fasta_file.endswith(".fa"):
        pass
    elif fasta_file.endswith(".fasta"):
        pass
    else:
        E.warn("are you sure this is a fasta file?")

    command = '''
    sailfish index --transcripts %s --out %s --threads %i --kmerSize %i
    ''' % (fasta_file, outdir, threads, kmer)

    os.system(command)


def runSailfishQuant(fasta_index, fastq_files, output_dir,
                     paired=False, library="ISF", threads=4,
                     gene_gtf=None):
    '''
    Wrapper for sailfish quant command
    '''

    decompress = False
    if len(fastq_files) > 1:
        if fastq_files[0].endswith(".gz"):
            decompress = True
        else:
            pass
    else:
        if fastq_files[0].endswith(".gz"):
            decompress = True
        else:
            pass

    # check output directory is an absolute path
    if os.path.isabs(output_dir):
        pass
    else:
        out_dir = os.path.abspath(output_dir)

    states = []
    command = " sailfish quant --index %s -l %s  -o %s " % (fasta_index,
                                                            library,
                                                            output_dir)

    states.append(command)

    if threads:
        states.append(" --threads %i " % threads)
    else:
        pass

    if gene_gtf:
        states.append(" --geneMap %s " % gene_gtf)
    else:
        pass

    # sailfish does not handle compress files natively,
    # need to decompress on the fly with advanced
    # bash syntax
    if decompress and paired:
        first_mates = tuple([fq for fq in fastq_files if re.search("fastq.1.gz",
                                                                   fq)])
        fstr_format = " ".join(["%s" for hq in first_mates])
        fdecomp_format = fstr_format % first_mates
        decomp_first = " -1 <( zcat %s )" % fdecomp_format

        states.append(decomp_first)

        second_mates = tuple([sq for sq in fastq_files if re.search("fastq.2.gz",
                                                                    sq)])
        sstr_format = " ".join(["%s" for aq in second_mates])
        sdecomp_format = sstr_format % second_mates
        decomp_second = " -2 <( zcat %s )" % sdecomp_format

        states.append(decomp_second)

    elif decompress and not paired:
        first_mates = tuple([fq for fq in fastq_files if re.search("fastq.gz",
                                                                   fq)])
        fstr_format = " ".join(["%s" for sq in first_mates])
        fdecomp_format = fstr_format % first_mates
        decomp_first = " -r <( zcat %s )" % fdecomp_format

        states.append(decomp_first)

    elif paired and not decompress:
        first_mates = tuple([fq for fq in fastq_files if re.search("fastq.1",
                                                                   fq)])
        fstr_format = " ".join(["%s" for sq in first_mates])
        fdecomp_format = fstr_format % first_mates
        decomp_first = " -1 %s " % fdecomp_format

        states.append(decomp_first)

        second_mates = tuple([sq for sq in fastq_files if re.search("fastq.2",
                                                                    sq)])
        sstr_format = " ".join(["%s" for aq in second_mates])
        sdecomp_format = sstr_format % second_mates
        decomp_second = " -2 %s " % sdecomp_format

        states.append(decomp_second)

    statement = " ".join(states)

    # subprocess cannot handle process substitution
    # therefore needs to be wrapped in /bin/bash -c '...'
    # for bash to interpret the substitution correctly
    process = subprocess.Popen(statement, shell=True,
                               executable="/bin/bash")

    stdout, stderr = process.communicate()

    if process.returncode != 0:
        raise OSError(
            "-------------------------------------------\n"
            "Child was terminated by signal %i: \n"
            "The stderr was \n%s\n%s\n"
            "-------------------------------------------" %
            (-process.returncode, stderr, statement))


def runKallistoIndex(fasta_file, outfile, kmer=31):
    '''
    Wrapper for kallisto index
    '''

    if fasta_file.endswith(".fa"):
        pass
    elif fasta_file.endswith(".fasta"):
        pass
    else:
        E.warn("are you sure this is a fasta file?")

    command = "kallisto index --index=%s  %s" % (outfile,
                                                 fasta_file)

    os.system(command)


def runKallistoQuant(fasta_index, fastq_files, output_dir,
                     bias=False, bootstrap=None,
                     seed=1245, threads=None, plaintext=False):
    '''
    Wrapper for kallisto quant command
    '''

    if len(fastq_files) > 1:
        fastqs = " ".join(fastq_files)
    else:
        fastqs = fastq_files

    # check output directory is an absolute path
    if os.path.isabs(output_dir):
        pass
    else:
        out_dir = os.path.abspath(output_dir)

    states = []
    command = " kallisto quant --index=%s --output-dir=%s" % (fasta_index,
                                                              output_dir)
    states.append(command)

    if bias:
        states.append(" --use-bias ")
    else:
        pass

    if bootstrap:
        states.append(" --bootstrap=%i --seed=%i " % (bootstrap,
                                                      seed))
    else:
        pass

    if plaintext:
        states.append(" --plaintext ")
    else:
        pass

    if threads:
        states.append(" --threads=%i " % threads)
    else:
        pass

    states.append(" %s " % fastqs)

    statement = " ".join(states)

    # need to rename output files to conform to input/output
    # pattern as required.  Default name is abundance*.txt
    # when using plaintext output
    # kaliisto requires an output directory - create many small
    # directories, one for each file.
    # then extract the abundance.txt file and rename using the
    # input/output pattern

    os.system(statement)


def main(argv=None):
    """script main.
    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option("-t", "--test", dest="test", type="string",
                      help="supply help")

    parser.add_option("--program", dest="program", type="choice",
                      choices=["kallisto", "sailfish"],
                      help="use either kallisto or sailfish, "
                      "for alignment-free quantification")

    parser.add_option("--method", dest="method", type="choice",
                      choices=["make_index", "quant"],
                      help="method of kallisto to run")

    parser.add_option("--index-fasta", dest="fa_index", type="string",
                      help="multi-fasta to use to make index for kallisto")

    parser.add_option("--index-file", dest="index_file", type="string",
                      help="kallisto index file to use for quantificaiton")

    parser.add_option("--use-bias", dest="bias", action="store_true",
                      help="use kallisto's bias correction")

    parser.add_option("--bootstraps", dest="bootstrap", type="int",
                      help="number of bootstraps to apply to quantification")

    parser.add_option("--seed", dest="seed", type="int",
                      help="seed number for random number genration "
                      "and bootstrapping")

    parser.add_option("--just-text", dest="text_only", action="store_true",
                      help="only output files in plain text, not HDF5")

    parser.add_option("--library-type", dest="library", type="choice",
                      choices=["ISF", "ISR", "IU", "MSF", "MSR", "MU",
                               "OSF", "OSR", "OU", "SR", "SF", "U"],
                      help="sailfish fragment library type code")

    parser.add_option("--paired-end", dest="paired", action="store_true",
                      help="data are paired end")

    parser.add_option("--kmer-size", dest="kmer", type="int",
                      help="kmer size to use for index generation")

    parser.add_option("--gene-gtf", dest="gene_gtf", type="string",
                      help="GTF file containing transcripts and gene "
                      "identifiers to calculate gene-level estimates")

    parser.add_option("--threads", dest="threads", type="int",
                      help="number of threads to use for kallisto "
                      "quantificaion")

    parser.add_option("--output-directory", dest="outdir", type="string",
                      help="directory to output transcript abundance "
                      "estimates to")

    parser.add_option("--output-file", dest="outfile", type="string",
                      help="output filename")

    parser.set_defaults(paired=False)

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv)

    if options.method == "make_index":
        if options.program == "kallisto":
            runKallistoIndex(fasta_file=options.fa_index,
                             outfile=options.outfile,
                             kmer=options.kmer)
        elif options.program == "sailfish":
            runSailfishIndex(fasta_file=options.fa_index,
                             outdir=options.outdir,
                             threads=options.threads,
                             kmer=options.kmer)
        else:
            E.warn("program not recognised, exiting.")

    elif options.method == "quant":
        infiles = argv[-1]
        qfiles = infiles.split(",")
        # make the output directory if it doesn't exist
        if os.path.exists(options.outdir):
            pass
        else:
            os.system("mkdir %s" % options.outdir)

        if options.program == "kallisto":
            runKallistoQuant(fasta_index=options.index_file,
                             fastq_files=qfiles,
                             output_dir=options.outdir,
                             bias=options.bias,
                             bootstrap=options.bootstrap,
                             seed=options.seed,
                             threads=options.threads,
                             plaintext=options.text_only)
        elif options.program == "sailfish":
            infiles = argv[-1]
            qfiles = infiles.split(",")
            runSailfishQuant(fasta_index=options.index_file,
                             fastq_files=qfiles,
                             output_dir=options.outdir,
                             paired=options.paired,
                             library=options.library,
                             threads=options.threads,
                             gene_gtf=options.gene_gtf)

        else:
            E.warn("program not recognised, exiting.")
    else:
        pass

    # write footer and output benchmark information.
    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''
cgat_rebuild_extensions.py - rebuild all cython extensions
==========================================================

:Tags: Python

Purpose
-------

This script rebuilds all cython extensions in the source directory.

Some scripts in the repository make use of ``pyximport`` to compile
associated cython scripts with embedded C code. Theses scripts are
automatically re-compiled if the script has changed, but this process
can fail if:

   * the script is executed on a machine without a C-compiler
   * some underlying libraries have changed.

Thus, it is safer to rebuild all scripts on a machine with a C compiler
before running a script in production on a cluster, where not all nodes
might be fully configured for compilation.

Usage
-----

Example::

   python cgat_rebuild_extensions.py

Type::

   python cgat_rebuild_extensions.py --help

for command line help.

Command line options
--------------------

'''

import os
import sys
import glob

import cgatcore.experiment as E


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $",
                            usage=globals()["__doc__"])

    parser.add_option("-i", "--test-option", dest="test_option", type="string",
                      help="test option [default=%default].")

    parser.set_defaults(
        test_option="test"
    )

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv)

    files = glob.glob(os.path.join(os.path.dirname(__file__), "*.pyx"))

    # do sth
    ninput, nskipped, noutput = 0, 0, 0

    for f in files:
        E.info("rebuilding %s" % f)
        ninput += 1
        prefix, suffix = os.path.splitext(f)
        for ext in (".c", ".pyxbldc"):
            try:
                os.remove(prefix + ext)
            except OSError:
                pass

        dirname, basename = os.path.split(prefix)
        assert basename.startswith("_")

        scriptname = os.path.join(dirname, basename[1:]) + ".py"
        if not os.path.exists(scriptname):
            E.warn("script %s does not exist - skipped" % scriptname)
            nskipped += 1
            continue

        E.info("compiling %s" % scriptname)
        os.system("%s %s --help > /dev/null" % (sys.executable, scriptname))

        noutput += 1

    E.info("ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput, nskipped))

    # write footer and output benchmark information.
    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''fasta2stats - extract statistics from fasta file
===================================================

This tool outputs the number of sequences and total length of
sequences. It works on uncompressed and compressed files and will make
use of a samtools faidx index if it exists.

Command line options
--------------------

'''
import os
import sys
import numpy
import pysam

import cgatcore.experiment as E
import cgatcore.iotools as iotools


def main(argv=None):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-f", "--fasta", dest="input_filename_fasta",
        type="string",
        help="filename with fasta sequences. ")

    parser.add_option(
        "-o", "--output-filename-sequences", dest="output_filename_sequences",
        type="string",
        help="output per sequence information to filename")

    parser.set_defaults(
        input_filename_fasta=None,
    )

    (options, args) = E.start(parser, argv=argv)

    if len(args) > 0:
        options.input_filename_fasta = args[0]

    sequence_pairs = []

    if options.input_filename_fasta != "-" and os.path.exists(
            options.input_filename_fasta + ".fai"):
        has_index = 1
        fastafile = pysam.FastaFile(options.input_filename_fasta)
        sequence_pairs = list(zip(fastafile.references, fastafile.lengths))
    else:
        has_index = 0
        iterator = pysam.FastxFile(options.input_filename_fasta)
        for record in iterator:
            sequence_pairs.append(
                (record.name,
                 len(record.sequence)))

    lengths = numpy.array([x[1] for x in sequence_pairs])

    options.stdout.write("\t".join((
        "has_index", "nsequences", "total_length", "min_length",
        "max_length", "median_length", "mean_length")) + "\n")

    if len(lengths) > 0:
        options.stdout.write("\t".join(map(str, (
            has_index,
            len(sequence_pairs),
            lengths.sum(),
            lengths.min(),
            lengths.max(),
            numpy.median(lengths),
            lengths.mean()))) + "\n")
    else:
        options.stdout.write("\t".join(map(str, (
            has_index,
            len(sequence_pairs),
            0,
            "",
            "",
            "",
            ""))) + "\n")

    if options.output_filename_sequences:
        with iotools.open_file(options.output_filename_sequences, "w") as outf:
            outf.write("name\tlength\n")
            outf.write(
                "\n".join(["\t".join(map(str, x)) for x in sequence_pairs]) + "\n")

    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''
split_file.py - split a file into parts
=======================================

:Tags: Python

Purpose
-------

.. todo::

   describe purpose of the script.

Usage
-----

Example::

   python split_file.py --help

Type::

   python split_file.py --help

for command line help.

Command line options
--------------------

'''
import sys
import re
import string
import os
import getopt
import cgatcore.experiment as E
import cgatcore.iotools as iotools

USAGE = """python %s < stdin > stdout

split a file into chunks.

OPTIONS:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-r, --split-regex               split at regular expression
-a, --after                     split after match
-s, --skip                      do not echo match
-p, --pattern-output            pattern of output files (has to contain %s)
-c, --column=                   split according to column
-m, --map=                      split according to map
-d, --dry-run                   echo files that would be created,
                                but do not create any.
-e, --header-names                    add header to each file
-r, --remove-key                remove key column
-append                         append data to existing files.
--pattern-identifier            if given, use this pattern to extract
                                id from column.
--chunk-size                    Number of matching records in each output file
--version                       output version information
""" % (sys.argv[0], "s")


def CreateOpen(file, mode="w", dry_run=False, header=None):
    """open file. Check first, if directory exists.
    """

    if dry_run:
        print("# opening file %s" % file)
        return iotools.open_file("/dev/null", mode)

    if mode in ("w", "a"):
        dirname = os.path.dirname(file)
        if dirname and not os.path.exists(dirname):
            os.makedirs(dirname)

    if os.path.exists(file):
        existed = True
    else:
        existed = False

    f = iotools.open_file(file, mode)

    if header and not existed:
        f.write(header + "\n")

    return f


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    param_long_options = [
        "verbose=", "help", "split-regex=", "after", "pattern-output=", "skip",
        "column=", "map=", "dry-run",
        "header", "remove-key", "append", "pattern-identifier=", "version",
        "chunk-size="]

    param_short_options = "v:hr:ap:sc:dek"

    param_loglevel = 1
    param_split_at_regex = None
    param_after = None
    param_skip = None
    param_pattern_output = "%s.chunk"
    param_split_column = None
    param_filename_map = None
    param_dry_run = False
    param_header = False
    param_remove_key = False
    param_append = "w"
    param_pattern_identifier = None
    param_chunk_size = 1

    try:
        optlist, args = getopt.getopt(sys.argv[1:],
                                      param_short_options,
                                      param_long_options)

    except getopt.error as msg:
        print(USAGE, msg)
        sys.exit(1)

    for o, a in optlist:
        if o in ("-v", "--verbose"):
            param_loglevel = int(a)
        elif o in ("--version", ):
            print("version=")
            sys.exit(0)
        elif o in ("-h", "--help"):
            print(USAGE)
            sys.exit(0)
        elif o in ("-r", "--split-regex"):
            param_split_at_regex = re.compile(a)
        elif o in ("-a", "--after"):
            param_after = 1
        elif o in ("-s", "--skip"):
            param_skip = 1
        elif o in ("-p", "--pattern-output"):
            param_pattern_output = a
        elif o in ("-c", "--column"):
            param_split_column = int(a) - 1
        elif o in ("-m", "--map"):
            param_filename_map = a
        elif o in ("-d", "--dry-run"):
            param_dry_run = True
        elif o in ("-e", "--header-names"):
            param_header = True
        elif o in ("-r", "--remove-key"):
            param_remove_key = True
        elif o == "--append":
            param_append = "a"
        elif o == "--pattern-identifier":
            param_pattern_identifier = re.compile(a)
        elif o == "--chunk-size":
            param_chunk_size = int(a)

    print(E.GetHeader())
    print(E.GetParams())

    mymap = {}
    if param_filename_map:
        infile = iotools.open_file(param_filename_map, "r")
        for line in infile:
            if line[0] == "#":
                continue
            data = line[:-1].split("\t")[:2]
            mymap[data[0]] = data[1]

    filenames = set()
    found = set()
    ninput, noutput = 0, 0

    if param_split_column is not None:

        header = None
        files = {}
        for line in sys.stdin:

            if line[0] == "#":
                continue

            ninput += 1

            if param_header:
                if not header:
                    header = line[:-1]
                    continue
            else:
                header = None

            data = line[:-1].split("\t")

            try:
                key = data[param_split_column]
            except ValueError:
                continue

            if param_pattern_identifier:
                key = param_pattern_identifier.search(key).groups()[0]

            if mymap:
                if key in mymap:
                    key = mymap[key]
                else:
                    continue

            found.add(key)

            filename = re.sub("%s", key, param_pattern_output)
            filenames.add(filename)

            if filename not in files:

                # reset if too many files are open
                if len(files) > 1000:
                    if param_loglevel >= 1:
                        print("# resetting all files.")
                        sys.stdout.flush()

                    for f in list(files.values()):
                        f.close()
                    files = {}

                files[filename] = CreateOpen(
                    filename, "a", param_dry_run, header)

            if param_remove_key:
                del data[param_split_column]
                files[filename].write(string.join(data, "\t") + "\n")
            else:
                files[filename].write(line)

            noutput += 1

        for f in list(files.values()):
            f.close()

    else:
        file_id = 0

        filename = re.sub("%s", str(file_id), param_pattern_output)
        outfile = CreateOpen(filename, param_append, param_dry_run)
        nlines = 0

        header = param_header
        split = 0

        for line in sys.stdin:

            if param_split_at_regex and param_split_at_regex.search(line[:-1]):
                split += 1

            if split == param_chunk_size:
                if param_after:
                    nlines += 1
                    outfile.write(line)
                if nlines > 0:
                    outfile.close()
                    file_id += 1
                    filename = re.sub("%s", str(file_id), param_pattern_output)
                    outfile = CreateOpen(
                        filename, param_append, param_dry_run, header)
                    filenames.add(filename)
                    split = 0

                nlines = 0
                if param_after or param_skip:
                    continue

            outfile.write(line)
            nlines += 1

        outfile.close()

    if param_loglevel >= 1:
        sys.stdout.write(
            "# ninput=%i, noutput=%i, nfound=%i, nnotfound=%i, nfiles=%i\n" % (
                ninput,
                noutput,
                len(found),
                len(set(mymap).difference(found)),
                len(filenames)))

    print(E.GetFooter())

if __name__ == "__main__":
    sys.exit(main(sys.argv))



"""Per position statistics from a BAM file
==========================================

Compute per-position statistics.

Methods
-------

read-variant:
    Output for each position the bases of the aligned reads.

read-list:
    Output for each position the reads that are aligned to this
    position.

depth-vcf:
    Base coverage at position (excluding deletions). Note that
    the output of this method is in :term:`VCF` format.

coverage-vcf:
    Read coverage at position (including reads that have a gap
    at site). Note that the output of this method is in :term:`VCF`
    format.

barcode:

"""

import sys
import os
import json
import re
import pandas
import pysam
import cgatcore.experiment as E


def generate_from_bed(bam_file, bed_file, **kwargs):
    for bed in bed_file.fetch(parser=pysam.asBed()):
        for v in bam_file.pileup(bed.contig, bed.start, bed.end,
                                 **kwargs,
                                 truncate=True):
            yield v


def generate_from_bam(bam_file, **kwargs):
    for v in bam_file.pileup(**kwargs):
        yield v


def generate_from_region(bam_file, region, **kwargs):
    for v in bam_file.pileup(region=region, **kwargs):
        yield v


def main(argv=None):

    if argv is None:
        argv = sys.argv

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-i", "--input-fastq-file", dest="input_fastq_file", type="string",
        help="input fastq file. "
        "[%default]")

    parser.add_option(
        "-m", "--method", dest="method", type="choice",
        choices=("read-variant", "depth-vcf", "read-list", "coverage-vcf", "barcode"),
        help="method to apply [%default]")

    parser.add_option(
        "-e", "--input-bed", dest="input_bed_file", type="string",
        help="input file with intervals. Tab-delimited file of intervals "
        "in bed format to restrict analysis to. [%default]")

    parser.add_option(
        "-r", "--region-string", dest="region_string", type="string",
        help="region string. Only apply method in specified region. "
        "[%default]")

    parser.add_option(
        "-f", "--reference-fasta-file", dest="reference_fasta_file",
        help="reference genomic sequence in fasta format. "
        "[%default]")

    parser.add_option(
        "--min-base-quality", dest="min_base_quality", type="int",
        help="minimum base quality for barcode analysis. "
        "[%default]")

    parser.add_option(
        "-s", "--stepper", dest="stepper", type="choice",
        choices=("nofilter", "samtools", "all"))

    parser.set_defaults(
        method="read-variant",
        reference_fasta_file=None,
        input_bed_file=None,
        regex_sample_name="([^/]+).bam",
        stepper="nofilter",
        min_base_quality=13,
        region_string=None)

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv, add_output_options=True)

    pysam_in = pysam.AlignmentFile(args[0], "rb")

    if options.input_bed_file:
        if not os.path.exists(options.input_bed_file):
            raise OSError("input bed file {} does not exist".format(
                options.input_bed_file))
        bed_in = pysam.TabixFile(options.input_bed_file)
    else:
        bed_in = None

    if options.region_string is not None:
        itr = generate_from_region(pysam_in, options.region,
                                   stepper=options.stepper,
                                   min_base_quality=options.min_base_quality)
    elif bed_in is not None:
        itr = generate_from_bed(pysam_in, bed_in,
                                stepper=options.stepper,
                                min_base_quality=options.min_base_quality)
    else:
        itr = generate_from_bam(pysam_in,
                                stepper=options.stepper,
                                min_base_quality=options.min_base_quality)

    reference_fasta = pysam.FastaFile(options.reference_fasta_file)

    outf = options.stdout
    counter = E.Counter()

    if options.method == "read-variant":
        outf.write("chromosome\tposition\tref\ttypes\n")

        for pileupcolumn in itr:
            counter.positions_pileup += 1
            reference_base = reference_fasta.fetch(
                pileupcolumn.reference_name,
                pileupcolumn.reference_pos,
                pileupcolumn.reference_pos + 1)
            matches = []
            bases = set()
            for read in pileupcolumn.pileups:
                qpos = read.query_position
                if qpos is not None:
                    base = read.alignment.query_sequence[qpos]
                else:
                    base = "-"

                matches.append((base,
                                read.alignment.query_name))
                bases.add(base)

            bases = list(bases)
            if len(bases) == 1:
                counter.position_noninformative += 1
                if bases[0] == reference_base:
                    counter.position_reference += 1
                continue

            counter.position_informative += 1

            d = {}
            for base in bases:
                d[base] = ",".join([x[1] for x in matches if x[0] == base])

            outf.write("{}\t{}\t{}\t{}\n".format(
                pileupcolumn.reference_name,
                pileupcolumn.reference_pos,
                reference_base,
                json.dumps(d)))

    elif options.method in ("depth-vcf", "coverage-vcf"):
        if options.regex_sample_name:
            sample_name = re.search(options.regex_sample_name, args[0]).groups()[0]
        else:
            sample_name = "unknown"

        outf.write("##fileformat=VCFv4.1\n")
        outf.write("##FORMAT=<ID=GT,Number=1,Type=String,"
                   "Description=\"Genotype\">\n")
        outf.write("##FORMAT=<ID=DP,Number=1,Type=Integer,"
                   "Description=\"Genotype\">\n")
        outf.write(
            "#CHROM\tPOS\tID\tREF\tALT\tQUAL\t"
            "FILTER\tINFO\tFORMAT\t{}\n".format(sample_name))

        is_depth = options.method == "depth-vcf"

        for idx, pileupcolumn in enumerate(itr):

            if idx % 1000 == 0:
                E.info("processed {} positions".format(idx))

            reference_base = reference_fasta.fetch(
                pileupcolumn.reference_name,
                pileupcolumn.reference_pos,
                pileupcolumn.reference_pos + 1).upper()

            if reference_base == 'A':
                alt_base = 'C'
            else:
                alt_base = 'A'

            if is_depth:
                n = sum([1 for x in pileupcolumn.pileups if not (x.is_del or x.is_refskip)])
            else:
                n = pileupcolumn.n

            outf.write("{}\t{}\t.\t{}\t{}\t.\tPASS\t.\tGT:DP\t0/1:{}\n".format(
                pileupcolumn.reference_name,
                pileupcolumn.reference_pos,
                reference_base,
                alt_base,
                n))

    elif options.method == "read-list":
        outf.write("chromosome\tposition\treference_base\tbase\tquality\tquery_name\n")

        for pileupcolumn in itr:
            reference_base = reference_fasta.fetch(pileupcolumn.reference_name,
                                                   pileupcolumn.reference_pos,
                                                   pileupcolumn.reference_pos + 1)
            matches = []
            for read in pileupcolumn.pileups:
                qpos = read.query_position
                if qpos is not None:
                    base = read.alignment.query_sequence[qpos]
                    quality = read.alignment.query_qualities[qpos]
                else:
                    base = "-"
                    quality = ""

                outf.write("{}\t{}\t{}\t{}\t{}\t{}\n".format(
                    pileupcolumn.reference_name,
                    pileupcolumn.reference_pos,
                    reference_base,
                    base,
                    quality,
                    read.alignment.query_name))

    elif options.method == "barcode":

        rows = []
        for c in itr:
            rows.append((c.reference_pos,
                         c.n,
                         "".join(c.get_query_sequences()),
                         pysam.qualities_to_qualitystring(c.get_query_qualities())))
        df = pandas.DataFrame.from_records(
            rows,
            columns=["pos", "gapped_depth", "bases", "qualities"])

        df["depth"] = df.bases.str.len()
        bases = ["A", "C", "G", "T"]
        for b in bases:
            df[b] = df.bases.str.upper().str.count(b)
        df["consensus"] = df[bases].idxmax(axis=1)
        df["consensus_counts"] = df.lookup(df.index, df.consensus)
        df["consensus_support"] = df.consensus_counts / df.depth
        df["offconsensus_counts"] = df.depth - df.consensus_counts
        df.loc[df.consensus_counts == 0, "consensus"] = "N"

        df.to_csv(outf, sep="\t", index=False)

    E.info(counter)
    # write footer and output benchmark information.
    E.stop()



'''output depth statistics for a BAM file.
'''

import collections
import subprocess
import re
import os
import shlex

import cgatcore.experiment as E
import cgatcore.iotools as iotools


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "--input-filename-fasta", dest="input_filename_fasta", type="string",
        help="filename with reference sequence in fasta format [%default]")

    parser.add_option(
        "--counting-mode", dest="counting_mode", type="choice",
        choices=("all", "pileup_defaults"),
        help="counting mode. all=all reads/bases. pileup-defaults= "
        "use default pileup thresholds. Options will be added to "
        "--mpileup-options. [%default].")

    parser.add_option(
        "--mpileup-options", dest="mpileup_options", type="string",
        help="pileup options to use [%default]")

    parser.set_defaults(
        mpileup_options="",
        counting_mode="all",
        input_filename_fasta=None,
        report_step=1000000,
    )

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv, add_output_options=True)

    bamfile = args[0]

    mpileup_options = options.mpileup_options

    if options.counting_mode == "all":
        mpileup_options += " -Q 0 -B -A"

    read_depth_histogram = collections.defaultdict(int)
    base_depth_histogram = collections.defaultdict(int)

    # deletions are marked by something like -2AA at the first
    # position and a '*' for subsequent positions
    rx_deletions = re.compile("([-][0-9]+|[*])")
    report_step = options.report_step
    npositions = 0

    samtools = iotools.which("samtools")

    statement = (
        "{samtools} mpileup "
        "-f {reference_fasta} "
        "{mpileup_options} "
        "{bamfile} ".format(
            samtools=samtools,
            reference_fasta=options.input_filename_fasta,
            mpileup_options=mpileup_options,
            bamfile=os.path.abspath(bamfile)))

    E.info("running the following statement: {}".format(statement))

    cmd_args = shlex.split(statement)
    proc = subprocess.Popen(
        cmd_args,
        shell=False,
        stderr=subprocess.PIPE,
        stdout=subprocess.PIPE,
        cwd=os.path.abspath(os.curdir))

    for line in proc.stdout:
        line = line.decode("utf-8")
        contig, pos, base, read_depth, info, qualities = line[:-1].split("\t")
        read_depth = int(read_depth)
        pos = int(pos)

        if pos % report_step == 0:
            E.info("working on {}: {}".format(contig, pos))

        ndeletions = len(rx_deletions.findall(info))
        base_depth = read_depth - ndeletions

        read_depth_histogram[read_depth] += 1
        base_depth_histogram[base_depth] += 1

    for line in proc.stderr:
        E.warn(line)

    keys = sorted(set(read_depth_histogram.keys()).union(
        base_depth_histogram.keys()))

    options.stdout.write("depth\tread_depth_positions\tbase_depth_positions\n")
    for key in keys:
        options.stdout.write("{}\t{}\t{}\n".format(
                key,
                read_depth_histogram[key],
                base_depth_histogram[key]))

    E.info("positions tested: {}".format(sum(read_depth_histogram.values())))
    E.stop()



'''combine_tables.py - join tables
===============================

:Tags: Python

Purpose
-------

This script reads several tab-separated tables and joins them into a
single one.

.. todo::

   * Rename to tables2table.py
   * Use pandas dataframes for fast IO and merging/joining

Usage
-----

The option ``--header-names`` sets the column titles explicitely. Add
``--skip-titles`` if you want to avoid echoing the original title in
the input files.


Example::

   python combine_tables.py --help

Type::

   python combine_tables.py --help

for command line help.

Command line options
--------------------

'''

import sys
from cgatcore.tables import main


if __name__ == '__main__':
    sys.exit(main(sys.argv))



'''
vcfstats_sqlite.py - reformat output of vcf-stats for database loading
======================================================================

:Tags: Python

Purpose
-------

create a csv separated file for loading into a database from 
output of vcf-stats utility in vcf-tools package.

Usage
-----

Example::

   python vcfstats_sqlite.py [files] > [outfile]

Type::

   python vcfstats_sqlite.py --help

for command line help.


Command line options
--------------------

'''
import os
import sys
import cgatcore.experiment as E
import cgatcore.iotools as iotools


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    parser = E.OptionParser(
        version="%prog version: $Id: vcfstats_sqlite.py 0001 2011-04-13 davids $", usage=globals()["__doc__"])

    (options, args) = E.start(parser)

    options.filenames = args

    if len(options.filenames) < 1:
        options.stdout.write("# Error: no vcf-stats files specified/found.")
        sys.exit(1)

    E.info("Parsing %i file(s)" % len(options.filenames))

    # set up output files
    vcf_file = iotools.open_file('vcfstats.txt', 'w')
    indel_file = iotools.open_file('indelstats.txt', 'w')
    snp_file = iotools.open_file('snpstats.txt', 'w')
    shared_file = iotools.open_file('sharedstats.txt', 'w')

    for fileno, filename in enumerate(options.filenames):

        prefix = os.path.basename(filename)
        trackname = prefix.replace(".vcfstats", "")

        if os.path.exists(filename):
            lines = [x for x in iotools.open_file(filename, "r").readlines()]
        else:
            lines = []

        if len(lines) == 0:
            options.stdout.write(
                "# Error: empty vcf-stats file found: $(filename)s")
            sys.exit(1)
        else:
            E.info("File %i contains %i lines" % (fileno, len(lines)))
            vcf_stats = dict(track=trackname)
            snp_stats = dict(track=trackname)
            indel_stats = dict()
            shared_stats = dict()
            all_vars = False
            indels = False
            snps = False
            shared = False
            for i, line in enumerate(lines):
                line = line.strip()
                if line.find("'all'") > -1:
                    all_vars = True
                    E.info("Found 'all'")
                    continue

                if all_vars:
                    if line.find("=>") > -1:
                        fields = line.split("=>")
                        key = fields[0].strip().replace(
                            "'", "").replace(">", "_")
                        val = fields[1].strip().replace(",", "")
                    else:
                        key = "NA"
                        val = "NA"
                    if key == "indel" and val == "{":
                        indels = True
                        E.info("Found 'indels'")
                        continue
                    elif key == "snp" and val == "{":
                        snps = True
                        E.info("Found 'SNPs'")
                        continue
                    elif key == "shared" and val == "{":
                        shared = True
                        E.info("Found 'Shared'")
                        continue

                    if indels:
                        if line.find("}") > -1:
                            indels = False
                            E.info("Processed 'indels'")
                            continue
                        else:
                            indel_stats[key] = val
                    elif snps:
                        if line.find("}") > -1:
                            snps = False
                            E.info("Processed 'SNPs'")
                            continue
                        else:
                            snp_stats[key] = val
                    elif shared:
                        if line.find("}") > -1:
                            shared = False
                            E.info("Processed 'Shared'")
                            continue
                        else:
                            shared_stats[key] = val
                    elif key != "NA":
                        vcf_stats[key] = val

            # Ensure all keys are present
            allkeys = ["nalt_1", "nalt_2", "nalt_3", "nalt_4",
                       "nalt_5", "track", "count", "snp_count", "indel_count"]
            for k in allkeys:
                if k in vcf_stats:
                    continue
                else:
                    vcf_stats[k] = "0"

            # Write header (for first file only)
            if filename == options.filenames[0]:

                # Ensure keys are sorted
                srt = list(vcf_stats.keys())
                srt.sort()
                sep = ""
                for k in srt:
                    vcf_file.write("%s%s" % (sep, k))
                    sep = "\t"
                vcf_file.write("\n")

                indel_file.write("track\tindel_length\tindel_count\n")
                shared_file.write("track\tno_samples\tvar_count\n")

                sep = ""
                for k in snp_stats.keys():
                    snp_file.write("%s%s" % (sep, k))
                    sep = "\t"
                snp_file.write("\n")

            # Write data
            sep = ""
            srt = list(vcf_stats.keys())
            srt.sort()
            for k in srt:
                vcf_file.write("%s%s" % (sep, vcf_stats[k]))
                sep = "\t"
            vcf_file.write("\n")

            # Check all indel lengths are covered
            r = list(range(-20, 20, 1))
            for i in r:
                if str(i) in indel_stats:
                    continue
                else:
                    indel_stats[i] = "0"
            for k in indel_stats.keys():
                indel_file.write("%s\t%s\t%s\n" %
                                 (trackname, k, indel_stats[k]))

            for k in shared_stats.keys():
                shared_file.write("%s\t%s\t%s\n" %
                                  (trackname, k, shared_stats[k]))

            sep = ""
            for k in snp_stats.keys():
                snp_file.write("%s%s" % (sep, snp_stats[k]))
                sep = "\t"
            snp_file.write("\n")

    # close files
    vcf_file.close()
    indel_file.close()
    snp_file.close()

    E.stop()
    sys.exit(0)


if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''
csv_intersection.py - intersect two tables
======================================================

:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python csv_intersection.py --help

Type::

   python csv_intersection.py --help

for command line help.

Command line options
--------------------

'''
import sys
import cgatcore.experiment as E
import cgatcore.iotools as iotools
from cgatcore.csvutils import readTable
import csv
import hashlib


class UniqueBuffer:
    mKeys = {}

    def __init__(self, outfile):
        self.mOutfile = outfile

    def write(self, out):
        key = hashlib.md5(out).digest()
        if key not in self.mKeys:
            self.mKeys[key] = True
            self.mOutfile.write(out)


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    parser = E.OptionParser(
        version="%prog version: $Id: csv_intersection.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-u", "--unique", dest="unique", action="store_true",
                      help="output rows are uniq.")

    parser.set_defaults(
        remove=False,
        unique=False,
    )

    (options, args) = E.start(parser, add_csv_options=True)

    if len(args) != 2:
        raise ValueError("please specify two files to join")

    options.filename1, options.filename2 = args

    table1 = readTable(iotools.open_file(options.filename1, "r"))
    table2 = readTable(iotools.open_file(options.filename2, "r"))

    if options.unique:
        outfile = UniqueBuffer(sys.stdout)
    else:
        outfile = options.stdout

    # build new field list
    new_fields = []

    for x in options.join_fields1:
        new_fields.append(x)

    for x in fields1:
        if x not in options.join_fields1:
            new_fields.append(x)
        if x not in options.join_fields2:
            new_fields.append(x)

        writer = csv.DictWriter(outfile,
                                fields,
                                dialect=options.csv_dialect,
                                lineterminator=options.csv_lineterminator,
                                extrasaction='ignore')

    if len(lines) > 0:

        old_fields = lines[0][:-1].split("\t")

        if options.remove:
            fields = []
            for x in old_fields:
                if x not in input_fields:
                    fields.append(x)
        else:
            fields = input_fields

        reader = csv.DictReader(lines,
                                dialect=options.csv_dialect)

        print("\t".join(fields))

        first_row = True
        for row in reader:
            row = iotools.convertDictionary(row)
            writer.writerow(row)

    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''
clean.py - clean up output files from aborted runs
==================================================

:Tags: Python

Purpose
-------

This script checks one or more output files have they
have completed successfully. It will remove output files
for those jobs that are incomplete.

The script checks for the "job finished" tag at the
end of the file.

Usage
-----

Example::

   python clean.py --help

Type::

   python clean.py --help

for command line help.

Command line options
--------------------
'''

import os
import sys
import re
import glob
import os.path
import cgatcore.experiment as E


def getLastLine(filename, read_size=1024):
    """return last line of a file.
    """
    f = iotools.open_file(
        filename, 'rU')    # U is to open it with Universal newline support
    offset = read_size
    f.seek(0, 2)
    file_size = f.tell()
    if file_size == 0:
        return ""
    while 1:
        if file_size < offset:
            offset = file_size
        f.seek(-1 * offset, 2)
        read_str = f.read(offset)
        # Remove newline at the end
        if read_str[offset - 1] == '\n':
            read_str = read_str[:-1]
        lines = read_str.split('\n')
        if len(lines) >= 2:
            return lines[-1]
        if offset == file_size:   # reached the beginning
            return read_str
        offset += read_size
    f.close()


def checkPythonRuns(filename):
    """returns true if a python run is complete."""
    last_line = getLastLine(filename)
    return re.match("# job finished", last_line)


def isNewer(a, b):
    """return true if file a is newer than file b."""

    # get times of most recent access
    at = os.stat(a)[7]
    bt = os.stat(b)[7]

    return at > bt


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    parser = E.OptionParser(version="%prog version: $Id: clean.py 2782 2009-09-10 11:40:29Z andreas $",
                            usage=globals()["__doc__"])

    parser.add_option("-g", "--glob", dest="glob_pattern", type="string",
                      help="glob pattern to use for collecting files [%default].")

    parser.add_option("-n", "--dry-run", dest="dry_run", action="store_true",
                      help="only print out actions, do not execute them [%default].")

    parser.add_option("-f", "--file-pattern", dest="file_pattern", type="string",
                      help="only check files matching this pattern [%default].")

    parser.set_defaults(glob_pattern="data.dir",
                        file_pattern=".out",
                        check_completeness="python",
                        skip_dirs=[],
                        dry_run=False,
                        )

    (options, args) = E.start(parser,
                              add_pipe_options=True)

    if args:
        starts = args
    elif options.glob_pattern:
        starts = glob.glob(options.glob_pattern)
    else:
        starts = "."

    ndirs, nfiles, ndeleted = 0, 0, 0

    if options.check_completeness == "python":
        isComplete = checkPythonRuns

    rx = re.compile(options.file_pattern)

    for start in starts:
        for root, dirs, files in os.walk(start):

            ndirs += 1
            # exclude directories
            for dir in options.skip_dirs:
                if dir in dirs:
                    dirs.remove(dir)

            for filename in files:
                p = os.path.join(root, filename)
                if rx.search(filename) and not isComplete(p):
                    if options.dry_run:
                        options.stdlog.write("# removing file %s\n" % p)
                    else:
                        os.remove(p)
                    ndeleted += 1

    if options.loglevel >= 1:
        options.stdlog.write("# ndirs=%i, nfiles=%i, ndeleted=%i\n" %
                             (ndirs, nfiles, ndeleted))

    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''compute per-windows stats from a bam-file

Purpose
-------

This script takes a bam file as input and computes a few metrics by
iterating over the file. The metrics output are:

'''
import pysam

import cgatcore.experiment as E
from cgat.BamTools.bamtools import bam2stats_window_count


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "--region", dest="region", type="string",
        help="region to restrict analysis to [%default]")

    parser.add_option(
        "--window-size", dest="window_size", type="int",
        help="window size to use [%default]")

    parser.add_option(
        "--output-all-windows", dest="output_all_windows", action="store_true",
        help="output all windows. By default, windows without reads are skipped "
        "[%default]")

    parser.add_option(
        "--reference-fasta", "--input-filename-fasta",
        dest="input_filename_fasta", type="string",
        help="filename with reference sequence. If given, used to "
        "compute G+C content in windows [%default]")

    parser.set_defaults(
        force_output=False,
        region=None,
        output_all_windows=False,
        window_size=500,
        input_filename_fasta=None,
    )

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv, add_output_options=True)

    is_stdin = True
    if len(args) > 0:
        pysam_in = pysam.AlignmentFile(args[0], "rb")
        if args[0] != "-":
            is_stdin = False
    elif options.stdin == sys.stdin:
        pysam_in = pysam.AlignmentFile("-", "rb")
    else:
        pysam_in = pysam.AlignmentFile(options.stdin, "rb")
        if options.stdin != "-":
            is_stdin = False

    if options.input_filename_fasta:
        fasta = pysam.FastaFile(options.input_filename_fasta)
    else:
        fasta = None

    counts_df = bam2stats_window_count(
        pysam_in,
        region=options.region,
        window_size=options.window_size,
        fasta=fasta)

    if not options.output_all_windows:
        counts_df = counts_df[counts_df.alignments > 0]

    # add G+C content
    if fasta:
        counts_df["percent_gc"] = 100.0 * counts_df.bases_gc / (counts_df.bases_gc + counts_df.bases_at)
        counts_df.fillna(0, inplace=True)

    counts_df.to_csv(
        options.stdout,
        sep="\t")

    E.stop()



'''fastq2summary.py - compute summary stats for a fastq file
======================================================

:Tags: Genomics NGS Sequences FASTQ Annotation

Purpose
-------

This script iterates over a fastq file and outputs
summary statistics for the complete file

The output is a tab-delimited text file with the some of following columns
depending on the option specified:


+----------------+-----------------------------------------------------------+
|*Column*        |*Content*                                                  |
+----------------+-----------------------------------------------------------+
|reads           |total reads in file                                        |
+----------------+-----------------------------------------------------------+
|bases           |total bases in file                                        |
+----------------+-----------------------------------------------------------+
|mean_length     |mean read length                                           |
+----------------+-----------------------------------------------------------+
|median_length   |median read length                                         |
+----------------+-----------------------------------------------------------+
|mean_quality    |mean read quality                                          |
+----------------+-----------------------------------------------------------+
|median_quality  |median read quality                                        |
+----------------+-----------------------------------------------------------+
|nfailed         |number of bases below quality threshold                    |
+----------------+-----------------------------------------------------------+


Usage
-----

Example::

   python fastq2summary.py --guess-format=sanger < in.fastq > out.tsv

In this example we know that our data have quality scores formatted as
sanger. Given that illumina-1.8 quality scores are highly overlapping
with sanger, this option defaults to sanger qualities. In default mode
the script may not be able to distinguish highly overlapping sets of
quality scores.

Type::

   python fastq2summary.py --help

for command line help.

Command line options
--------------------

'''

import sys
import numpy as np
import cgatcore.experiment as E
import cgat.Fastq as Fastq


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "--guess-format", dest="guess_format", type="choice",
        choices=('sanger', 'solexa', 'phred64',
                 'illumina-1.8', 'integer'),
        help="The default behaviour of the script is to guess \
        the quality format of the input fastq file. The user \
        can specify the quality format of the input file using \
        the --format option. The script will use this format if \
        sequences qualities are ambiguous.[default=%default].")

    parser.add_option(
        "-f", "--target-format", dest="change_format",
        type="choice", choices=('sanger', 'solexa', 'phred64',
                                'illumina-1.8', 'integer'),
        help="The script guesses the quality format of the input \
        file and converts quality scores to the destination \
        format unless --format is specified [default=%default].")

    parser.set_defaults(
        change_format=None,
        guess_format=None,
        min_quality=10)

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv)

    if options.change_format:
        iterator = Fastq.iterate_convert(options.stdin,
                                         format=options.change_format,
                                         guess=options.guess_format)
    else:
        iterator = Fastq.iterate_guess(options.stdin,
                                       guess=options.guess_format)

    min_quality = options.min_quality
    number_of_reads = 0
    number_of_bases = 0
    read_lengths = []
    read_qualities = []
    bases_below_min = 0

    for record in iterator:
        number_of_reads += 1
        quals = record.toPhred()
        length_read = len(quals)
        number_of_bases += length_read
        bases_below_min += len([x for x in quals if x < min_quality])
        read_lengths.append(length_read)
        read_qualities.append(np.mean(quals))

    mean_length = round(np.mean(read_lengths), 2)
    median_length = round(np.median(read_lengths), 2)
    mean_quality = round(np.mean(read_qualities), 2)
    median_quality = round(np.median(read_qualities), 2)

    options.stdout.write(
        "reads\tbases\tmean_length\tmedian_length\tmean_quality\tmedian_quality\tnfailed\n")

    options.stdout.write(
        "%i\t%i\t%s\t%s\t%s\t%s\t%i\n" % (number_of_reads, number_of_bases,
                                          str(mean_length),
                                          str(median_length),
                                          str(mean_quality),
                                          str(median_quality),
                                          bases_below_min))
    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



'''
csvs2csv.py - join tables
=========================

:Tags: Python

Purpose
-------

This script reads several tab-separated tables and joins them.

.. note:: 
   working with multiple columns per table and sorting is
   not implemented correctly and likely to fail.

Usage
-----

Example::

   python combine_tables.py --help

Type::

   python combine_tables.py --help

for command line help.

Command line options
--------------------

'''
import sys
import re
import os
import glob

import cgatcore.experiment as E


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    parser = E.OptionParser(
        version="%prog version: $Id$",
        usage=globals()["__doc__"])

    parser.add_option(
        "-t", "--no-titles", dest="titles", action="store_false",
        help="no titles in input.")

    parser.add_option(
        "-i", "--skip-titles", dest="skip_titles", action="store_true",
        help="skip output of titles.")

    parser.add_option(
        "-m", "--missing-value", dest="missing_value", type="string",
        help="entry to use for missing values.")

    parser.add_option("--header-names", dest="headers", type="string",
                      help="add headers for files.")

    parser.add_option(
        "-c", "--columns", dest="columns", type="string",
        help="columns to use for joining. Multiple columns can be specified "
        "as a comma-separated list [default=%default].")

    parser.add_option(
        "-g", "--glob", dest="glob", type="string",
        help="wildcard expression for table names.")

    parser.add_option(
        "-s", "--sort-order", dest="sort", type="string",
        help="sort by column titles alphabetical|numeric|list of columns.")

    parser.add_option(
        "-e", "--merge-overlapping", dest="merge", action="store_true",
        help="simply merge tables without matching up rows. "
        "[default=%default].")

    parser.add_option(
        "--sort-keys", dest="sort_keys", type="choice",
        choices=("numeric", "alphabetic"),
        help="sort key columns by value.")

    parser.add_option(
        "--keep-empty", dest="ignore_empty", action="store_false",
        help="keep empty tables. The default is to ignore them.")

    parser.add_option(
        "--add-file-prefix", dest="add_file_prefix", action="store_true",
        help="add file prefix to columns headers in multi-column tables "
        "[default=%default]")

    parser.add_option(
        "--regex-filename", dest="regex_filename", type="string",
        help="pattern to apply to filename to build prefix "
        "[default=%default]")

    parser.set_defaults(
        titles=True,
        skip_titles=False,
        missing_value="na",
        headers=None,
        sort=None,
        glob=None,
        columns="1",
        sort_keys=False,
        merge=False,
        ignore_empty=True,
        add_file_prefix=False,
        regex_filename="(.*)"
    )

    (options, args) = E.start(parser)

    if options.headers:
        if "," in options.headers:
            options.headers = options.headers.split(",")
        else:
            options.headers = re.split("\s+", options.headers.strip())

    if options.sort and options.sort not in ("numeric", "alphabetic"):
        if "," in options.sort:
            options.sort = options.sort.split(",")
        else:
            options.sort = re.split("\s+", options.sort)

    if options.merge:
        options.columns = []
    else:
        options.columns = [int(x) - 1 for x in options.columns.split(",")]

    options.filenames = []

    if options.glob:
        options.filenames += glob.glob(options.glob)

    options.filenames += args

    if len(options.filenames) < 1:
        print(USAGE, "no tables specified/found.")
        sys.exit(1)

    if options.loglevel >= 1:
        options.stdlog.write("# combining %i tables.\n" %
                             len(options.filenames))
        sys.stdout.flush()
        if len(options.filenames) == 1:
            for line in iotools.open_file(options.filenames[0]):
                options.stdout.write(line)
            E.stop()
            sys.exit(0)

    if options.headers and options.headers[0] != "auto" and \
       len(options.headers) != len(options.filenames):
        raise "number of provided headers (%i) is not equal to number filenames (%i)." %\
              (len(options.headers), len(options.filenames))

    tables = []
    keys = {}
    sorted_keys = []
    sizes = {}
    if options.merge:
        titles = ["count"]
    else:
        titles = []

    for filename in options.filenames:

        prefix = os.path.basename(filename)

        if os.path.exists(filename):
            file = iotools.open_file(filename, "r")
            lines = [x for x in file if x[0] != "#"]

        else:
            lines = []

        if len(lines) == 0 and options.ignore_empty:
            continue

        table = {}
        sizes = {}
        max_size = 0
        ncolumns = 0

        if options.titles:
            data = lines[0][:-1].split("\t")
            if not titles:
                key = "-".join([data[x] for x in options.columns])
                titles = [key]
            for x in range(len(data)):
                if x in options.columns:
                    continue
                ncolumns += 1
                if options.add_file_prefix:
                    p = re.search(options.regex_filename, prefix).groups()[0]
                    titles.append("%s_%s" % (p, data[x]))
                else:
                    titles.append(data[x])

            del lines[0]
        else:
            ncolumns = 1

        n = 0
        for line in lines:
            data = line[:-1].split("\t")
            row_keys = [data[x] for x in options.columns]
            if options.sort_keys:
                if options.sort_keys == "numeric":
                    row_keys.sort(lambda x, y: cmp(float(x), float(y)))
                else:
                    row_keys.sort()
            if options.merge:
                key = n
            else:
                key = "-".join(row_keys)

            if key not in keys:
                sorted_keys.append(key)
                keys[key] = 1
                sizes[key] = 0

            max_size = max(len(data) - len(options.columns), max_size)
            table[key] = [data[x]
                          for x in [x for x in range(0, len(data)) if x not in options.columns]]
            n += 1

        # enter columns of "na" for empty tables.
        if max_size == 0:
            max_size = ncolumns

        tables.append((max_size, table))

    if len(tables) == len(titles) - 1:

        if options.headers:
            headers = ["bin"]
            if options.headers[0] == 'auto':
                for t in range(len(tables)):
                    headers.append(os.path.basename(options.filenames[t]))
                    headers += [""] * (tables[t][0] - 1)

            else:
                for t in range(len(tables)):
                    headers.append(options.headers[t])
                    headers += [""] * (tables[t][0] - 1)

            # use headers as titles, if headers is given and skip-titles is
            # turned on
            if options.titles and options.skip_titles:
                titles = headers
            else:
                # otherwise: print the headers out right away
                sys.stdout.write("\t".join(headers) + "\n")

        order = list(range(0, len(tables) + 1))

        if options.titles:

            if options.sort:
                sort_order = []

                if options.sort == "numeric":
                    t = list(zip(list(map(int, titles[1:])), list(range(1, len(titles) + 1))))
                    t.sort()

                    for tt in t:
                        sort_order.append(titles[tt[1]])

                elif options.sort == "alphabetical":
                    t = list(zip(titles[1:], list(range(1, len(titles) + 1))))
                    t.sort()

                    for tt in t:
                        sort_order.append(titles[tt[1]])
                else:
                    sort_order = options.sort

                map_title2pos = {}
                for x in range(1, len(titles)):
                    map_title2pos[titles[x]] = x

                order = [0, ]
                for x in sort_order:
                    if x in map_title2pos:
                        order.append(map_title2pos[x])

            else:
                order = list(range(0, len(titles)))

            sys.stdout.write(
                "\t".join([titles[order[x]] for x in range(len(titles))]))
            sys.stdout.write("\n")

        if options.sort_keys:
            if options.sort_keys:
                if options.sort_keys == "numeric":
                    sorted_keys.sort(lambda x, y: cmp(float(x), float(y)))
                else:
                    sorted_keys.sort()

        for key in sorted_keys:

            sys.stdout.write("%s" % key)

            for x in order[1:]:
                max_size, table = tables[x - 1]
                c = 0
                if key in table:
                    sys.stdout.write("\t")
                    sys.stdout.write("\t".join(table[key]))
                    c = len(table[key])

                assert(max_size == 1)

                sys.stdout.write(
                    "\t%s" % options.missing_value * (max_size - c))

            sys.stdout.write("\n")

    else:

        # for multi-column table, just write
        if options.titles:
            sys.stdout.write(
                "\t".join([titles[x] for x in range(len(titles))]))
            sys.stdout.write("\n")

        for key in sorted_keys:

            sys.stdout.write("%s" % key)

            for x in range(len(tables)):

                max_size, table = tables[x]
                c = 0
                if key in table:
                    sys.stdout.write("\t")
                    sys.stdout.write("\t".join(table[key]))
                    c = len(table[key])

                sys.stdout.write(
                    "\t%s" % options.missing_value * (max_size - c))

            sys.stdout.write("\n")

    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))



"""compare two bed files.
"""

import collections
import pysam
import sys
import cgatcore.experiment as E
import cgatcore.iotools as iotools
import quicksect


def get_size_bin(size, size_bins):
    bin = 0
    nsize_bins = len(size_bins)
    while bin < nsize_bins and size > size_bins[bin]:
        bin += 1
    return bin


def main(argv=sys.argv):

    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option(
        "-b", "--reference-bed-file", dest="reference_bed_file", type="string",
        help="reference bed file "
        "[%default]")

    parser.add_option(
        "-m", "--method", dest="method", type="choice",
        choices=("lvc-comparison", ),
        help="methods to apply [%default]")

    parser.set_defaults(
        method="lvc-comparison",
        reference_fasta_file=None,
        input_bed_file=None,
        size_bins=(1000, 10000, 100000),
        output_sets=True,
        region_string=None)

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv, add_output_options=True)

    reference_set = collections.defaultdict(
        quicksect.IntervalTree)

    E.info("reading reference bed file from {}".format(options.reference_bed_file))
    with iotools.open_file(options.reference_bed_file) as inf:
        for record in pysam.tabix_iterator(inf, pysam.asBed()):
            mm = reference_set[record.contig]
            mm.add(record.start,
                   record.end)
    E.info("read reference intervals on {} contigs: {}".format(
            len(list(reference_set.keys())), ",".join(list(reference_set.keys()))))

    if options.output_sets:
        output_tp = E.open_output_file("tp")
        output_fp = E.open_output_file("fp")
        output_fn = E.open_output_file("fn")
    else:
        output_tp = None
        output_fp = None
        output_fn = None

    if options.method == "lvc-comparison":
        c = E.Counter()

        found = set()
        counts = {}
        names = set()
        nsize_bins = len(options.size_bins)
        for bin in range(len(options.size_bins) + 1):
            counts[bin] = dict([(x, collections.defaultdict(int)) for x in
                                ("tp", "fn", "fp", "test", "truth")])

        for record in pysam.tabix_iterator(options.stdin, pysam.asBed()):
            if record.contig not in reference_set:
                c.ignored_no_contig += 1
                continue

            c.test += 1
            matches = reference_set[record.contig].search(record.start, record.end)
            size = record.end - record.start
            bin = get_size_bin(size, options.size_bins)

            if len(matches) == 0:
                c.fp += 1
                status = "fp"
                if output_fp:
                    output_fp.write(str(record) + "\n")
            elif len(matches) >= 1:
                c.tp += 1
                status = "tp"
                if output_tp:
                    output_tp.write(str(record) + "\n")
                # todo: overlap criteria

                # record found
                for match in matches:
                    found.add((record.contig, match.start, match.end))

            name = record.name.split(",")[0]
            names.add(name)
            counts[bin]["test"][name] += 1
            counts[bin][status][name] += 1

        outf = options.stdout

        with iotools.open_file(options.reference_bed_file) as inf:
            for record in pysam.tabix_iterator(inf, pysam.asBed()):
                c.truth += 1
                bin = get_size_bin(record.end - record.start, options.size_bins)
                counts[bin]["truth"]["all"] += 1

                key = (record.contig, record.start, record.end)
                if key not in found:
                    c.fn += 1
                    counts[bin]["fn"]["all"] += 1

        outf.write("\t".join(("category",
                              "size",
                              "test",
                              "tp",
                              "fp",
                              "truth",
                              "fn")) + "\n")

        for name in sorted(names):
            for bin in range(len(options.size_bins) + 1):
                if bin == len(options.size_bins):
                    size_bin = ">={}".format(options.size_bins[-1])
                else:
                    size_bin = "<{}".format(options.size_bins[bin])
                outf.write("\t".join(map(str, (
                                name,
                                size_bin,
                                counts[bin]["test"][name],
                                counts[bin]["tp"][name],
                                counts[bin]["fp"][name],
                                counts[bin]["truth"]["all"],
                                counts[bin]["fn"]["all"],
                                ))) + "\n")

    E.info(str(c))
    E.stop()



'''
lca2table.py
====================================================

:Tags: Python

Purpose
-------

Summarise results of LCA analysis - from mtools lcamapper.sh

Usage
-----

Example::

   python lca2table.py < infile > outfile

Type::

   python lca2table.py --help

for command line help.

Command line options
--------------------

'''

import sys
import cgat.LCA as LCA
import cgatcore.experiment as E
import collections


def main(argv=None):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if argv is None:
        argv = sys.argv

    # setup command line parser
    parser = E.OptionParser(version="%prog version: $Id$",
                            usage=globals()["__doc__"])

    parser.add_option("-s", "--summarise", dest="summarise", type="choice",
                      choices=("level-counts", "taxa-counts", "individual"),
                      help="summarise the taxa counts - no. phyla etc")

    parser.add_option("--output-map", dest="output_map", action="store_true",
                      help="ouput map of taxonomy")

    # add common options (-h/--help, ...) and parse command line
    (options, args) = E.start(parser, argv=argv)

    if options.output_map:
        found = []
        options.stdout.write("""Domain\t \
        kingdom\t \
        phylum\t \
        class\t \
        order\t \
        family\t \
        genus\t \
        species\n""")
        # only output the mapping file - do not continue
        # summarise regardless of the specified options
        for lca in LCA.iterate(options.stdin):

            # if bacteria or archaea the kingdom will
            # be the domain
            if lca.domain == "Bacteria" or lca.domain == "Archaea":
                kingdom = lca.domain
            else:
                kingdom = lca.kingdom

            hierarchy = [lca.domain,
                         kingdom,
                         lca.phylum,
                         lca._class,
                         lca.order,
                         lca.family,
                         lca.genus,
                         lca.species]
            if hierarchy in found:
                continue
            else:
                found.append(hierarchy)
                options.stdout.write("\t".join(hierarchy) + "\n")
        return

    if options.summarise == "level-counts":
        level_counts = collections.defaultdict(set)
        total = 0
        nreads_domain = 0
        nreads_kingdom = 0
        nreads_kingdom_plus = 0
        nreads_phylum = 0
        nreads_phylum_plus = 0
        nreads_class = 0
        nreads_class_plus = 0
        nreads_order = 0
        nreads_order_plus = 0
        nreads_family = 0
        nreads_family_plus = 0
        nreads_genus = 0
        nreads_genus_plus = 0
        nreads_species = 0
        nreads_species_plus = 0
        nreads_subspecies = 0
        nreads_subspecies_plus = 0

        c = E.Counter()
        for lca in LCA.iterate(options.stdin):
            total += 1
            if lca.domain != "NA":
                nreads_domain += 1
                level_counts["domain"].add(lca.domain)
            else:
                c.kingdom_unmapped += 1

            if lca.kingdom != "NA":
                nreads_kingdom += 1
                level_counts["kingdom"].add(lca.kingdom)
            else:
                c.kingdom_unmapped += 1

            if lca.kingdom_plus != "NA":
                nreads_kingdom_plus += 1
                level_counts["kingdom+"].add(lca.kingdom_plus)
            else:
                c.kingdom_plus_unmapped += 1

            if lca.phylum != "NA":
                nreads_phylum += 1
                level_counts["phylum"].add(lca.phylum)
            else:
                c.phylum_unmapped += 1

            if lca.phylum_plus != "NA":
                nreads_phylum_plus += 1
                level_counts["phylum+"].add(lca.phylum_plus)
            else:
                c.phylum_plus_unmapped += 1

            if lca._class != "NA":
                nreads_class += 1
                level_counts["class"].add(lca._class)
            else:
                c.class_unmapped += 1

            if lca._class_plus != "NA":
                nreads_class_plus += 1
                level_counts["class+"].add(lca._class_plus)
            else:
                c.class_plus_unmapped += 1

            if lca.order != "NA":
                nreads_order += 1
                level_counts["order"].add(lca.order)
            else:
                c.order_unmapped += 1

            if lca.order_plus != "NA":
                nreads_order_plus += 1
                level_counts["order+"].add(lca.order_plus)
            else:
                c.order_plus_unmapped += 1

            if lca.family != "NA":
                nreads_family += 1
                level_counts["family"].add(lca.family)
            else:
                c.family_unmapped += 1

            if lca.family != "NA":
                nreads_family_plus == 1
                level_counts["family+"].add(lca.family_plus)
            else:
                c.family_plus_unmapped += 1

            if lca.genus != "NA":
                nreads_genus += 1
                level_counts["genus"].add(lca.genus)
            else:
                c.genus_unmapped += 1

            if lca.genus_plus != "NA":
                nreads_genus_plus == 1
                level_counts["genus+"].add(lca.genus_plus)
            else:
                c.genus_plus_unmapped += 1

            if lca.species != "NA":
                nreads_species += 1
                level_counts["species"].add(lca.species)
            else:
                c.species_unmapped += 1

            if lca.species_plus != "NA":
                nreads_species_plus += 1
                level_counts["species+"].add(lca.species_plus)
            else:
                c.species_plus_unmapped += 1

            # removed subspecies mapping for the time
            # being
            
            # if lca.subspecies != "NA":
            #     nreads_subspecies += 1
            #     level_counts["subspecies"].add(lca.subspecies)
            # else:
            #     c.subspecies_unmapped += 1

            # if lca.subspecies_plus != "NA":
            #     nreads_subspecies_plus += 1
            #     level_counts["subspecies+"].add(lca.subspecies_plus)
            # else:
            #     c.subspecies_plus_unmapped += 1

        options.stdout.write("\t".join(["ndomain",
                                        "nkingdom",
                                        "nkingdom+",
                                        "nphylum",
                                        "nphylum+",
                                        "nclass",
                                        "nclass+",
                                        "norder",
                                        "norder+",
                                        "nfamily",
                                        "nfamily+",
                                        "ngenus",
                                        "ngenus+",
                                        "nspecies",
                                        "nspecies+",
                                        "nseqkingdom",
                                        "nseqkingdom+",
                                        "nseqphylum",
                                        "nseqphylum+",
                                        "nseqclass",
                                        "nseqclass+",
                                        "nseqorder",
                                        "nseqorder+",
                                        "nseqfamily",
                                        "nseqfamily+",
                                        "nseqgenus",
                                        "nseqgenus+",
                                        "nseqspecies",
                                        "nseqspecies+"]) + "\n")

        options.stdout.write("\t".join(map(
            str, [len(level_counts["domain"]),
                  len(level_counts["kingdom"]),
                  len(level_counts["kingdom+"]),
                  len(level_counts["phylum"]),
                  len(level_counts["phylum+"]),
                  len(level_counts["class"]),
                  len(level_counts["class+"]),
                  len(level_counts["order"]),
                  len(level_counts["order+"]),
                  len(level_counts["family"]),
                  len(level_counts["family+"]),
                  len(level_counts["genus"]),
                  len(level_counts["genus+"]),
                  len(level_counts["species"]),
                  len(level_counts["species+"]),
                  nreads_domain,
                  nreads_kingdom,
                  nreads_phylum,
                  nreads_phylum_plus,
                  nreads_class,
                  nreads_class_plus,
                  nreads_order,
                  nreads_order_plus,
                  nreads_family,
                  nreads_family_plus,
                  nreads_genus,
                  nreads_genus_plus,
                  nreads_species,
                  nreads_species_plus])) + "\n")
    elif options.summarise == "taxa-counts":
        unmapped = collections.defaultdict(int)
        total = 0
        taxa_counts = {"domain": collections.defaultdict(int),
                       "kingdom": collections.defaultdict(int),
                       "kingdom+": collections.defaultdict(int),
                       "phylum": collections.defaultdict(int),
                       "phylum+": collections.defaultdict(int),
                       "class": collections.defaultdict(int),
                       "class+": collections.defaultdict(int),
                       "order": collections.defaultdict(int),
                       "order+": collections.defaultdict(int),
                       "family": collections.defaultdict(int),
                       "family+": collections.defaultdict(int),
                       "genus": collections.defaultdict(int),
                       "genus+": collections.defaultdict(int),
                       "species": collections.defaultdict(int),
                       "species+": collections.defaultdict(int)}

        c = E.Counter()
        for lca in LCA.iterate(options.stdin):
            total += 1
            if lca.domain != "NA":
                taxa_counts["domain"][lca.domain] += 1
            else:
                c.kingdom_unmapped += 1
                unmapped["domain"] += 1
            if lca.kingdom != "NA":
                taxa_counts["kingdom"][lca.kingdom] += 1
            else:
                c.kingdom_unmapped += 1
                unmapped["kingdom"] += 1
            if lca.kingdom_plus != "NA":
                taxa_counts["kingdom+"][lca.kingdom_plus] += 1
            else:
                c.kingdom_plus_unmapped += 1
                unmapped["kingdom+"] += 1
            if lca.phylum != "NA":
                taxa_counts["phylum"][lca.phylum] += 1
            else:
                c.phylum_unmapped += 1
                unmapped["phylum"] += 1
            if lca.phylum_plus != "NA":
                taxa_counts["phylum+"][lca.phylum_plus] += 1
            else:
                c.phylum_plus_unmapped += 1
                unmapped["phylum+"] += 1
            if lca._class != "NA":
                taxa_counts["class"][lca._class] += 1
            else:
                c.class_unmapped += 1
                unmapped["class"] += 1
            if lca._class_plus != "NA":
                taxa_counts["class+"][lca._class_plus] += 1
            else:
                c.class_plus_unmapped += 1
                unmapped["class+"] += 1
            if lca.order != "NA":
                taxa_counts["order"][lca.order] += 1
            else:
                c.order_unmapped += 1
                unmapped["order"] += 1
            if lca.order_plus != "NA":
                taxa_counts["order+"][lca.order_plus] += 1
            else:
                c.order_plus_unmapped += 1
                unmapped["order+"] += 1
            if lca.family != "NA":
                taxa_counts["family"][lca.family] += 1
            else:
                c.family_unmapped += 1
                unmapped["family"] += 1
            if lca.family_plus != "NA":
                taxa_counts["family+"][lca.family_plus] += 1
            else:
                c.family_plus_unmapped += 1
                unmapped["family+"] += 1
            if lca.genus != "NA":
                taxa_counts["genus"][lca.genus] += 1
            else:
                c.genus_unmapped += 1
                unmapped["genus"] += 1
            if lca.genus_plus != "NA":
                taxa_counts["genus+"][lca.genus_plus] += 1
            else:
                c.genus_plus_unmapped += 1
                unmapped["genus+"] += 1
            if lca.species != "NA":
                taxa_counts["species"][lca.species] += 1
            else:
                c.species_unmapped += 1
                unmapped["species"] += 1
            if lca.species_plus != "NA":
                taxa_counts["species+"][lca.species_plus] += 1
            else:
                c.species_plus_unmapped += 1
                unmapped["species+"] += 1

        options.stdout.write("level\ttaxa\tcount\tproportion\trpm\n")
        for level, taxa_count in sorted(taxa_counts.items()):
            total_level = total - unmapped[level]
            for taxa, count in sorted(taxa_count.items()):
                options.stdout.write("\t".join(
                    [level,
                     taxa,
                     str(count),
                     "{:.8}".format(float(count)/total_level),
                     "{:.8}".format(float(count)/(float(total_level)/1000000))])
                                     + "\n")

        E.info(c)

    elif options.summarise == "individual":
        # each read is output with its respective
        # taxon assignments
        options.stdout.write("\t".join(["id",
                                        "domain",
                                        "kingdom",
                                        "kingdom+",
                                        "phylum",
                                        "phylum+",
                                        "class",
                                        "class+",
                                        "order",
                                        "order+",
                                        "family",
                                        "family+",
                                        "genus",
                                        "genus+",
                                        "species",
                                        "species+"]) + "\n")
        for lca in LCA.iterate(options.stdin):
            options.stdout.write("\t".join([lca.identifier,
                                            lca.domain,
                                            lca.kingdom,
                                            lca.kingdom_plus,
                                            lca.phylum,
                                            lca.phylum_plus,
                                            lca._class,
                                            lca._class_plus,
                                            lca.order,
                                            lca.order_plus,
                                            lca.family,
                                            lca.family_plus,
                                            lca.genus,
                                            lca.genus_plus,
                                            lca.species,
                                            lca.species_plus]) + "\n")

    # write footer and output benchmark information.
    E.stop()

if __name__ == "__main__":
    sys.exit(main(sys.argv))
